#include "fsck.h"

static pthread_mutex_t con_mutex = PTHREAD_MUTEX_INITIALIZER;

static uint64_t *same;
size_t n_same;

static void con_lock(void)
{
	int r;
	if ((r = pthread_mutex_lock(&con_mutex)))
		fprintf(stderr, "nvfsck: pthread_mutex_lock failed: %s\n", strerror(r)), exit(8);
}

static void con_unlock(void)
{
	int r;
	if ((r = pthread_mutex_unlock(&con_mutex)))
		fprintf(stderr, "nvfsck: pthread_mutex_unlock failed: %s\n", strerror(r)), exit(8);
}

void fsck_log(const char *msg, ...)
{
	va_list va;
	va_start(va, msg);
	con_lock();
	fprintf(stderr, "nvfsck: ");
	vfprintf(stderr, msg, va);
	fprintf(stderr, "\n");
	con_unlock();
	va_end(va);
}

void fsck_internal_error(const char *msg, ...)
{
	va_list va;
	va_start(va, msg);
	fprintf(stderr, "nvfsck internal error: ");
	vfprintf(stderr, msg, va);
	fprintf(stderr, "\n");
	va_end(va);
	abort();
}

static int get_response(void)
{
	int c, d;
	c = getchar();
	d = c;
	while (d != '\n' && d != EOF)
		d = getchar();
	return c;
}

bool fsck_query(int level, const char *msg, const char *question, ...)
{
	int c;
	va_list va;
	con_lock();
print_again:
	va_start(va, question);
	fprintf(stderr, "nvfsck: ");
	vfprintf(stderr, msg, va);
	fprintf(stderr, "\n");
	va_end(va);

	if (level > Q_PREEN && preen)
		goto ret_false;
	if (level == Q_NOAUTO && yes)
		goto ret_false;
	if (level != Q_NOAUTO && all)
		goto ret_true;

	if (ro)
		goto ret_false;
	if (yes)
		goto ret_true;

	if (level != Q_NOAUTO) {
		size_t s;
		for (s = 0; s < n_same; s++)
			if (!strcmp(msg, (void *)(size_t)same[s]))
				goto ret_true;
	}

	fprintf(stderr, "%s (y/n%s) ", question, level != Q_NOAUTO ? "/same/all" : "");

	c = get_response();
	if (c == EOF)
		goto ret_false;
	if ((c & 0xdf) == 'Y')
		goto ret_true;
	if ((c & 0xdf) == 'N')
		goto ret_false;
	if ((c & 0xdf) == 'S') {
		if (level != Q_NOAUTO)
			append_array(&same, &n_same, (size_t)msg);
		goto ret_true;
	}
	if ((c & 0xdf) == 'A') {
		all = 1;
		goto ret_true;
	}
	goto print_again;

ret_false:
	uncorrected_errors = 1;
	con_unlock();
	return false;

ret_true:
	modified = 1;
	con_unlock();
	return true;
}

void init_fsck_log(void)
{
	init_array(&same, &n_same);
}

void done_fsck_log(void)
{
	fsck_free(same);
}
